/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#if HAVE_CONFIG_H
#  include <config.h>
#endif

#include "private.h" 
#include <stdarg.h>	
#include <stdio.h>
#include <errno.h> 

#if HAVE_ENDIAN_H
# include <endian.h>
#else
# ifdef HAVE_PPC_ENDIAN_H
#  include <ppc/endian.h>
# else
#  error No endian defined!
# endif
#endif

#include <assert.h>

#define LOW_BYTE 0x00FF
#define HIGH_BYTE 0xFF00


// this is our logging function
// at the moment all it does is call printf
void log_it(const char *fmt, va_list ap, int err) {
	vprintf(fmt, ap) ;
	if (err) 
		printf("\nError is %s\n" , strerror(err)  ) ;
	printf("\n") ;
}

void err_warn(const char * fmt , ... ) {
	va_list ap;
	va_start(ap, fmt) ;

	log_it(fmt,ap, 0) ;

	va_end(ap) ;

}

void err_fatal(const char *fmt, ... ) {
	int err ;
	va_list ap;
	err = errno ;

	va_start(ap, fmt) ;

	log_it(fmt, ap, err) ;

	va_end(ap) ;
	exit(1) ;
}

void *Malloc(int len) {
	void *ptr = malloc(len) ;

	if(ptr == NULL) 
		err_fatal("malloc failed") ;
	return ptr ;
}

void *Calloc(size_t nmemb, int len) {
	void *ptr = calloc(nmemb, len) ;

	if(ptr == NULL) 
		err_fatal("calloc failed") ;
	return ptr ;
}


// this should be re-done as a macro
// i have no idea wtf is going on here, it seems to work though
int16_t bytes_to_short(uint8_t h, uint8_t l){
	int16_t o ;

#ifdef HAVE_ENDIAN_H
	if(BYTE_ORDER == __LITTLE_ENDIAN) {
#else
	if(BYTE_ORDER == LITTLE_ENDIAN) {
#endif
		o = (h << 8) | (l & LOW_BYTE) ;
		
	} else {
		// this is untested - danger
		o = (l << 8) + h ;
	}
	assert(short_get_low_byte(o) == l) ;
	assert(short_get_high_byte(o) == h) ;
	return o;
	
}

// converts 4 bytes in big endian order to a 32 bit int
int32_t nbytes_to_32(uint8_t bytes[4]) {
	int32_t o = 0;

//	printf(" input is l[0] : 0x%hhx , l[1] : 0x%hhx, l[2] : 0x%hhx, l[3] : 0x%hhx\n", bytes[0],bytes[1],bytes[2],bytes[3] ) ;
	
	o = (bytes[0] << 24) | (bytes[1] << 16) | (bytes[2] << 8) | bytes[3] ;

//	printf(" output is 0x%x\n", o) ;
	return o;
	
}

void int_to_bytes(int data, uint8_t *bytes) {
		
		bytes[3] = (data & 0x000000FF) ;
		bytes[2] = (data & 0x0000FF00) >> 8 ;
		bytes[1] = (data & 0x00FF0000) >> 16 ;
		bytes[0] = (data & 0xFF000000) >> 24 ;
			
}


// check for endian issues here
uint8_t short_get_low_byte(int16_t i){

	return LOW_BYTE & i ;
}

// check for endian issues here
uint8_t short_get_high_byte(int16_t i) {

	return (HIGH_BYTE & i) >> 8 ;
}


